# One dimensional optimization algorithms for function optimization
from __future__ import annotations

import sys
from typing import Tuple, Callable

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal
import torch

from .support import HistoryGSS, HistorySPI, HistoryBrent, update_history_brent


def golden_section_search(function: Callable[[float | torch.Tensor], float],
                          bounds: Tuple[float, float],
                          epsilon: float = 1e-5,
                          type_optimization: Literal['min', 'max'] = 'min',
                          max_iter: int = 500,
                          verbose: bool = False,
                          keep_history: bool = False) -> Tuple[float | torch.Tensor, HistoryGSS]:
    """
    Returns the optimal point and history using the Golden Section search [2]_

    :math:`\\rule{125mm}{0.7pt} \\\\`
    :math:`\\textbf{Constant: } \\displaystyle \\varphi = \\frac{(1 + \\sqrt{5})}{2} \\\\`
    :math:`\\textbf{Input: } f(x) - \\text{ function }; a, b - \\text{ left and right bounds };
    \\varepsilon - \\text{ precision } \\\\`
    :math:`\\rule{125mm}{0.3pt}\\\\`

    :math:`\\text{while } |a - b| > \\varepsilon: \\\\`
    :math:`\\qquad \\displaystyle x_1 = b - \\frac{b - a}{\\varphi} \\\\`
    :math:`\\qquad \\displaystyle x_2 = a + \\frac{b - a}{\\varphi} \\\\`
    :math:`\\qquad \\text{if }  f(x_1) > f(x_2): \\\\`
    :math:`\\qquad \\qquad a = x_1 \\\\`
    :math:`\\qquad \\text{else}: \\\\`
    :math:`\\qquad \\qquad b = x_2 \\\\`
    :math:`\\rule{125mm}{0.3pt}\\\\`
    :math:`\\textbf{Return: } \\displaystyle \\frac{a+b}{2} \\\\`
    :math:`\\rule{125mm}{0.7pt} \\\\`

    .. note::
        If optimization fails golden_section_search will return the last point


    .. rubric:: References

    ..  [2] Press, William H., Teukolsky, S. A., Vetterling, W. T., & Flannery, B. P. (2007).
        Numerical Recipes with Source Code CD-ROM 3rd Edition: The Art of Scientific Computing (3rd ed.).
        Cambridge University Press. p.492-496

    :math:`\\rule{125mm}{0.2pt} \\\\`

    :param function: callable that depends on the first positional argument. Other arguments are passed through kwargs
    :param bounds: tuple with two numbers. This is left and right bound optimization. [a, b]
    :param epsilon: optimization accuracy
    :param type_optimization: 'min' / 'max' - type of required value
    :param max_iter: maximum number of iterations
    :param verbose: flag of printing iteration logs
    :param keep_history: flag of return history
    :return: tuple with point and history.

    .. rubric:: Examples

    .. code-block:: python3

        >>> def func(x): return 2.71828 ** (3 * x) + 5 * 2.71828 ** (-2 * x)
        >>> point, data = golden_section_search(func, (-10, 10), type_optimization='min', keep_history=True)
    """
    phi: float = (1 + 5 ** 0.5) / 2

    type_optimization = type_optimization.lower().strip()
    assert type_optimization in ['min', 'max'], 'Invalid type optimization. Enter "min" or "max"'

    a: float = bounds[0]
    b: float = bounds[1]
    if keep_history:
        history: HistoryGSS = {'iteration': [0],
                               'middle_point': [(a + b) / 2],
                               'f_value': [function((a + b) / 2)],
                               'left_point': [a],
                               'right_point': [b]}

    else:
        history: HistoryGSS = {'iteration': [], 'middle_point': [], 'f_value': [], 'left_point': [], 'right_point': []}

    if verbose:
        print(f'Iteration: {0} \t|\t point = {(a + b) / 2 :0.3f} '
              f'\t|\t f(point) = {function((a + b) / 2): 0.3f}')

    try:
        for i in range(1, max_iter):
            x1: float = b - (b - a) / phi
            x2: float = a + (b - a) / phi

            if type_optimization == 'min':
                if function(x1) > function(x2):
                    a = x1
                else:
                    b = x2
            else:
                if function(x1) < function(x2):
                    a = x1
                else:
                    b = x2

            middle_point: float = (a + b) / 2
            if verbose:
                print(f'Iteration: {i} \t|\t point = {middle_point :0.3f} '
                      f'\t|\t f(point) = {function(middle_point): 0.3f}')

            if keep_history:
                history['iteration'].append(i)
                history['middle_point'].append(middle_point)
                history['f_value'].append(function(middle_point))
                history['left_point'].append(a)
                history['right_point'].append(b)

            if abs(x1 - x2) < epsilon:
                print('Searching finished. Successfully. code 0')
                return middle_point, history
        else:
            middle_point = (a + b) / 2
            print('Searching finished. Max iterations have been reached. code 1')
            return middle_point, history

    except Exception as e:
        print('Error with optimization. code 2')
        raise e


def successive_parabolic_interpolation(function: Callable[[float | torch.Tensor], float],
                                       bounds: Tuple[float, float],
                                       epsilon: float = 1e-5,
                                       type_optimization: Literal['min', 'max'] = 'min',
                                       max_iter: int = 500,
                                       verbose: bool = False,
                                       keep_history: bool = False) -> Tuple[float | torch.Tensor, HistorySPI]:
    """
    Returns the optimal point and history using the Successive parabolic interpolation algorithm [3]_

    :math:`\\rule{125mm}{0.7pt} \\\\`

    .. math::
        :label: eq1

        \\displaystyle x_{i+1}=x_{i}+ \\frac{1}{2}\\left[\\frac{\\left(x_{i-1}-x_{i}\\right)^{2}
        \\left(f_{i}-f_{i-2}\\right)+
        \\left(x_{i-2}-x_{i}\\right)^{2}\\left(f_{i-1}-f_{i}\\right)}{\\left(x_{i-1}-x_{i}\\right)
        \\left(f_{i}-f_{i-2}\\right)+\\left(x_{i-2}-x_{i}\\right)\\left(f_{i-1}-f_{i}\\right)}\\right]\\\\


    :math:`\\rule{125mm}{0.3pt}\\\\`

    :math:`\\textbf{Input: } f(x) - \\text{ function}; a, b - \\text{ left and right bounds};
    \\varepsilon - \\text{ precision } \\\\`
    :math:`\\rule{125mm}{0.3pt}\\\\`

    :math:`\\displaystyle x_0 = a, \\ f_0 = f(x_0); \\qquad  x_1 = b, \\ f_1 = f(x_1); \\qquad x_2 = \\displaystyle
    \\frac{a+b}{2}, \\ f_2 = f(x_2)\\\\`
    :math:`\\text{while } |x_{i+1}-x_{i}| \\geq \\varepsilon` or :math:`|f(x_{i+1})-f(x_{i})| \\geq \\varepsilon:\\\\`
    :math:`\\qquad \\displaystyle x_0, x_1, x_2` so that :math:`f_2 \\leq f_1 \\leq f_0\\\\`
    :math:`\\qquad \\displaystyle \\text{Calculate } x_{i + 1} \\text{with the formula }`  :eq:`eq1` :math:`\\\\`

    :math:`\\rule{125mm}{0.3pt}\\\\`
    :math:`\\textbf{Return: } \\displaystyle x_{i+1} \\\\`
    :math:`\\rule{125mm}{0.7pt} \\\\`


    ..  [3] Press, William H., Teukolsky, S. A., Vetterling, W. T., & Flannery, B. P. (2007).
        Numerical Recipes with Source Code CD-ROM 3rd Edition: The Art of Scientific Computing (3rd ed.).
        Cambridge University Press. p.496-499

    :math:`\\rule{125mm}{0.2pt} \\\\`

    :param function: callable that depends on the first positional argument. Other arguments are passed through kwargs
    :param bounds: tuple with two numbers. This is left and right bound optimization. [a, b]
    :param epsilon: optimization accuracy
    :param type_optimization: 'min' / 'max' - type of required value
    :param max_iter: maximum number of iterations
    :param verbose: flag of printing iteration logs
    :param keep_history: flag of return history
    :return: tuple with point and history.

    .. rubric:: Examples

    .. code-block:: python3

        >>> def func1(x): return x ** 3 - x ** 2 - x
        >>> successive_parabolic_interpolation(func1, (0, 1.5), verbose=True)
        Iteration: 0	|	x2 = 0.750	|	f(x2) = -0.891
        Iteration: 1	|	x2 = 0.850	|	f(x2) = -0.958
        Iteration: 2	|	x2 = 0.961	|	f(x2) = -0.997
        Iteration: 3	|	x2 = 1.017	|	f(x2) = -0.999
        Iteration: 4	|	x2 = 1.001	|	f(x2) = -1.000
        ...

        >>> def func2(x): return - (x ** 3 - x ** 2 - x)
        >>> successive_parabolic_interpolation(func2, (0, 1.5), type_optimization='max', verbose=True)
        Iteration: 0	|	x2 = 0.750	|	f(x2) = 0.891
        Iteration: 1	|	x2 = 0.850	|	f(x2) =  0.958
        Iteration: 2	|	x2 = 0.961	|	f(x2) =  0.997
        Iteration: 3	|	x2 = 1.017	|	f(x2) =  0.999
        ...
    """
    type_optimization = type_optimization.lower().strip()
    assert type_optimization in ['min', 'max'], 'Invalid type optimization. Enter "min" or "max"'

    if type_optimization == 'max':
        type_opt_const = -1
    else:
        type_opt_const = 1

    history: HistorySPI = {'iteration': [], 'f_value': [], 'x0': [], 'x1': [], 'x2': []}
    x0, x1 = bounds[0], bounds[1]
    x2 = (x0 + x1) / 3 * 2
    f0 = type_opt_const * function(x0)
    f1 = type_opt_const * function(x1)
    f2 = type_opt_const * function(x2)
    f_x: dict = {x0: f0, x1: f1, x2: f2}
    count_upd_point: int = 0  # if the numerator or denominator is zero, we will choose a new point. max of 100 times
    x2, x1, x0 = sorted([x0, x1, x2], key=lambda x: f_x[x])

    if keep_history:
        history['iteration'].append(0)
        history['f_value'].append(type_opt_const * f2)
        history['x0'].append(x0)
        history['x1'].append(x1)
        history['x2'].append(x2)

    if verbose:
        print(f'Iteration: {0}\t|\tx2 = {x2:0.3f}\t|\tf(x2) = {type_opt_const * f2: 0.3f}')

    try:
        for i in range(1, max_iter):
            f0, f1, f2 = f_x[x0], f_x[x1], f_x[x2]
            p = (x1 - x2) ** 2 * (f2 - f0) + (x0 - x2) ** 2 * (f1 - f2)
            q = 2 * ((x1 - x2) * (f2 - f0) + (x0 - x2) * (f1 - f2))

            if p == 0 or q == 0:
                if count_upd_point >= 100:
                    print('Select an another initial state. Numerator or Denominator is zero. code 2')
                    return x2, history
                else:
                    count_upd_point += 1
                    x2 = float(torch.rand(1) * (abs(x0 - x1)) * min(x0, x1))
                    f_x[x2] = function(x2)
                    continue

            x_new = x2 + p / q

            if not bounds[0] <= x_new <= bounds[1]:
                print('Searching finished. Out of bounds. code 1')
                return x2, history

            f_new = type_opt_const * function(x_new)
            f_x[x_new] = f_new
            previous_xs = [x0, x1, x2]

            if f_new < f2:
                x0, f0 = x1, f1
                x1, f1 = x2, f2
                x2, f2 = x_new, f_new

            elif f_new < f1:
                x0, f0 = x1, f1
                x1, f1 = x_new, f_new

            elif f_new < f0:
                x0, f0 = x_new, f_new

            if verbose:
                print(f'Iteration: {i}\t|\tx2 = {x2:0.3f}\t|\tf(x2) = {type_opt_const * f2: 0.3f}')

            if keep_history:
                history['iteration'].append(i)
                history['f_value'].append(type_opt_const * f2)
                history['x0'].append(x0)
                history['x1'].append(x1)
                history['x2'].append(x2)

            # In addition, check the criterion when the points don't change
            change_flag = max(map(lambda x, y: abs(x - y), [x0, x1, x2], previous_xs)) < epsilon
            if abs(x1 - x2) < epsilon and abs(f1 - f2) < epsilon or change_flag:
                print('Searching finished. Successfully. code 0')
                return x2, history

        else:
            print('Searching finished. Max iterations have been reached. code 1')
            return x2, history

    except Exception as e:
        print('Error with optimization. code 2')
        raise e


def brent(function: Callable[[float | torch.Tensor], float],
          bounds: Tuple[float, float],
          epsilon: float = 1e-5,
          type_optimization: Literal['min', 'max'] = 'min',
          max_iter: int = 500,
          verbose: bool = False,
          keep_history: bool = False) -> Tuple[float | torch.Tensor, HistoryBrent]:
    """
    Returns the optimal point and history using the Brent's algorithm [1]_.

    :math:`\\rule{125mm}{0.7pt} \\\\`
    :math:`\\textbf{Input: } f(x) - \\text{ function }; a, b - \\text{ left and right bounds };
    \\varepsilon - \\text{ precision } \\\\`
    :math:`\\rule{125mm}{0.3pt}\\\\`

    :math:`\\displaystyle \\varphi = \\frac{(1 + \\sqrt{5})}{2} \\\\`
    :math:`\\displaystyle x_{least} = a + \\varphi \\cdot (b - a) \\\\`
    :math:`\\displaystyle x_{new} = x_{least} \\\\`
    :math:`\\displaystyle tolerance = \\varepsilon \\cdot | x_{least}| + 10^{-9} \\\\`

    :math:`\\text{while }\\displaystyle |x_{least} - \\frac{a+b}{2}| > 2 \\cdot tolerance - \\frac{b-a}{2} :\\\\`

    :math:`\\qquad  \\text{if }\\displaystyle |x_{new} - x_{least}| > tolerance:\\\\`
    :math:`\\qquad \\qquad \\text{calculate parabolic } remainder \\text{ by formula }` :eq:`eq1` :math:`\\\\`
    :math:`\\qquad \\text{if } \\displaystyle remainder < previous \\ remainder \\ \\& \\
    x_{least} + remainder \\in (a, b):\\\\`

    :math:`\\qquad \\qquad  \\text{use  ``parabolic" } \\displaystyle remainder\\\\`

    :math:`\\qquad \\text{else:}\\\\`
    :math:`\\qquad \\qquad \\text{make  ``golden"  } \\displaystyle remainder\\\\`
    :math:`\\qquad \\qquad \\text{use ``golden" } \\displaystyle remainder\\\\`
    :math:`\\qquad \\displaystyle x_{new} = x_{least} + remainder\\\\`

    :math:`\\rule{125mm}{0.3pt}\\\\`
    :math:`\\textbf{Return: } \\displaystyle x_{least} \\\\`
    :math:`\\rule{125mm}{0.7pt} \\\\`

    .. rubric:: References

    .. [1] Brent, R. P., Algorithms for Minimization Without Derivatives. Englewood Cliffs, NJ: Prentice-Hall,
        1973 pp.72-80

    :math:`\\rule{125mm}{0.2pt} \\\\`

    :param function: callable that depends on the first positional argument. Other arguments are passed through kwargs
    :param bounds: tuple with two numbers. This is left and right bound optimization. [a, b]
    :param epsilon: optimization accuracy
    :param type_optimization: 'min' / 'max' - type of required value
    :param max_iter: maximum number of iterations
    :param verbose: flag of printing iteration logs
    :param keep_history: flag of return history
    :return: tuple with point and history.


    .. rubric:: Examples

    .. code-block:: python3

        >>> brent(lambda x: x ** 3 - x ** 2 - x, (0,2), verbose=True)[0]
        iteration 0	x = 0.763932,	f(x) = -0.901699	type : initial
        iteration 1	x = 0.763932,	f(x) = -0.901699	type : golden
        iteration 2	x = 0.763932,	f(x) = -0.901699	type : golden
        iteration 3	x = 0.944272,	f(x) = -0.993962	type : golden
        iteration 4	x = 0.944272,	f(x) = -0.993962	type : golden
        iteration 5	x = 0.999120,	f(x) = -0.999998	type : parabolic
        iteration 6	x = 0.999223,	f(x) = -0.999999	type : parabolic
        iteration 7	x = 0.999223,	f(x) = -0.999999	type : golden
        iteration 8	x = 0.999992,	f(x) = -1.000000	type : parabolic
        iteration 9	x = 1.000002,	f(x) = -1.000000	type : parabolic
        iteration 10	x = 1.000002,	f(x) = -1.000000	type : golden
        iteration 11	x = 1.000002,	f(x) = -1.000000	type : parabolic
        Searching finished. Successfully. code 0
        1.0000016327177492


    """

    type_optimization = type_optimization.lower().strip()
    assert type_optimization in ['min', 'max'], 'Invalid type optimization. Enter "min" or "max"'

    if type_optimization == 'max':
        type_opt_const = -1
    else:
        type_opt_const = 1

    gold_const = (3 - 5 ** 0.5) / 2
    remainder = 0.0  # p / q when we calculate x_new

    # initial values
    a, b = sorted(bounds)
    x_largest = x_middle = x_least = a + gold_const * (b - a)
    f_largest = f_middle = f_least = type_opt_const * function(x_least)
    x_least: float | torch.Tensor

    history: HistoryBrent = {
        'iteration': [],
        'f_least': [],
        'f_middle': [],
        'f_largest': [],
        'x_least': [],
        'x_middle': [],
        'x_largest': [],
        'left_bound': [],
        'right_bound': [],
        'type_step': []
    }

    if keep_history:
        history = update_history_brent(
            history,
            [0, f_least, f_middle, f_largest, x_least, x_middle, x_largest, a, b, 'initial']
        )

    if verbose:
        print(f'iteration 0\tx = {x_least:0.6f},\tf(x) = {f_least:0.6f}\ttype : initial')

    for i in range(1, max_iter + 1):
        middle_point = (a + b) / 2
        tolerance = epsilon * abs(x_least) + 1e-9  # f is never evaluated at two points closer together than tolerance

        # Check stopping criterion
        if abs(x_least - middle_point) > 2 * tolerance - (b - a) / 2:
            p = q = previous_remainder = 0
            if abs(remainder) > tolerance:

                # fit parabola
                p = ((x_least - x_largest) ** 2 * (f_least - f_middle) -
                     (x_least - x_middle) ** 2 * (f_least - f_largest))

                q = 2 * ((x_least - x_largest) * (f_least - f_middle) -
                         (x_least - x_middle) * (f_least - f_largest))

                # change q sign to positive
                if q > 0:
                    p = -p
                else:
                    q = -q
                # r stores the previous value of remainder
                previous_remainder = remainder

            # Check conditions for parabolic step:
            # tol - x_new must not be close to x_least, so we check the step
            # previous_remainder - the value of p / q at the second-last cycle
            # |previous_remainder| > tol - is checked above
            # q != 0 - includes in next conditions
            # x_least + p / q in (a, b). New point in interval
            # p / q < previous(p / q) / 2. Control the divergence

            if abs(p) < 0.5 * abs(q * previous_remainder) and a * q < x_least * q + p < b * q:
                remainder = p / q
                x_new = x_least + remainder
                name_step = 'parabolic'

                # Check that f not be evaluated too close to a or b
                if x_new - a < 2 * tolerance or b - x_new < 2 * tolerance:
                    if x_least < middle_point:
                        remainder = tolerance
                    else:
                        remainder = -tolerance

            # If conditions above is false we do golden section step
            else:
                name_step = 'golden'
                if x_least < middle_point:
                    remainder = (b - x_least) * gold_const
                else:
                    remainder = (a - x_least) * gold_const

            # Check that f not be evaluated too close to x_least
            if abs(remainder) > tolerance:
                x_new = x_least + remainder
            elif remainder > 0:
                x_new = x_least + tolerance
            else:
                x_new = x_least - tolerance

            f_new = type_opt_const * function(x_new)

            # Update a, b, x_largest, x_middle, x_leas
            if f_new <= f_least:
                if x_new < x_least:
                    b = x_least
                else:
                    a = x_least

                x_largest = x_middle
                f_largest = f_middle

                x_middle = x_least
                f_middle = f_least

                x_least = x_new
                f_least = f_new

            else:
                if x_new < x_least:
                    a = x_new
                else:
                    b = x_new

                if f_new <= f_middle:
                    x_largest = x_middle
                    f_largest = f_middle

                    x_middle = x_new
                    f_middle = f_new

                elif f_new <= f_largest:
                    x_largest = x_new
                    f_largest = f_new

        else:
            print('Searching finished. Successfully. code 0')
            return x_least, history

        if keep_history:
            history = update_history_brent(
                history,
                [
                    i,
                    type_opt_const * f_least,
                    type_opt_const * f_middle,
                    type_opt_const * f_largest,
                    x_least,
                    x_middle,
                    x_largest,
                    a,
                    b,
                    name_step
                ]
            )

        if verbose:
            print(f'iteration {i}\tx = {x_least:0.6f},\tf(x) = {type_opt_const * f_least:0.6f}\ttype : {name_step}')

    else:
        print('Searching finished. Max iterations have been reached. code 1')
        return x_least, history



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['nueramic_mathml', 'nueramic_mathml.ml', 'nueramic_mathml.visualize']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.21.6,<2.0.0',
 'pandas>=1.3,<2.0',
 'plotly>=5.8.2,<6.0.0',
 'scikit-learn>=1.1.1,<2.0.0',
 'torch>=1.11.0,<2.0.0']

setup_kwargs = {
    'name': 'nueramic-mathml',
    'version': '0.1.11',
    'description': 'Math algorithms in ML on torch',
    'long_description': '.. -*- mode: rst -*-\n   \n|PyPi|_ |Python|_ |Download|_ |License|_ |RTD|_\n\n\n------\n\n.. |PyPi| image:: https://img.shields.io/pypi/v/nueramic-mathml?color=edf2f4&style=flat-square\n.. _PyPi: https://pypi.org/project/nueramic-mathml/\n\n\n.. |Python| image:: https://img.shields.io/pypi/pyversions/p?color=edf2f4&style=flat-square\n.. _Python: https://github.com/nueramic/mathml\n\n.. |Download| image:: https://img.shields.io/pypi/dm/nueramic-mathml?color=edf2f4&label=dowloads&style=flat-square\n.. _Download: https://pypi.org/project/nueramic-mathml/\n\n.. |License| image:: https://img.shields.io/github/license/nueramic/mathml?color=edf2f4&style=flat-square\n.. _License: https://github.com/nueramic/mathml\n\n.. |RTD| image:: https://img.shields.io/readthedocs/nueramic-mathml?color=edf2f4&style=flat-square\n.. _RTD: https://nueramic-mathml.readthedocs.io\n\n.. |Colab_1| image:: https://colab.research.google.com/assets/colab-badge.svg\n.. _Colab_1: https://colab.research.google.com/drive/19moQvDMK8kfTDYOGuRwEl06jdf_KXNMW?usp=sharing\n\n..\n    .. raw:: html\n\n       <p align="center">\n       <picture align="center">\n         <source width=150px" media="(prefers-color-scheme: dark)" srcset="docs/_static/nueramic-logo-cropped-white.svg">\n         <source width=150px" media="(prefers-color-scheme: light)" srcset="docs/_static/nueramic-logo-cropped-black.svg">\n         <img alt="two logos" src="docs/_static/nueramic-logo-cropped-black.svg">\n       </picture>\n       </p>\n\n\nNueramic MathML\n===============\nNueramic-mathml is a library for visualizing and logging the steps of optimization algorithms in machine learning. The project uses torch for calculations and plotly for visualization.\n\n.. code-block:: python\n\n    pip install nueramic-mathml\n\n\nQuick tour  |Colab_1|_\n======================\n\nOptimization\n------------------\nYou can minimize the functions and see a detailed description of each step. After minimizing, you have a history with complete logs.\nAlso available multidimensional optimisation.\n\n.. code-block:: python\n\n    def f(x): return x ** 3 - x ** 2 - x  # Minimum at x = 1\n    bounds = (0, 3)\n    one_optimize.golden_section_search(f, bounds, epsilon=0.01, verbose=True)[0]\n\n    Iteration: 0 \t|\t point = 1.500 \t|\t f(point) = -0.375\n    Iteration: 1 \t|\t point = 0.927 \t|\t f(point) = -0.990\n    Iteration: 2 \t|\t point = 1.281 \t|\t f(point) = -0.820\n    Iteration: 3 \t|\t point = 1.062 \t|\t f(point) = -0.992\n    Iteration: 4 \t|\t point = 0.927 \t|\t f(point) = -0.990\n    Iteration: 5 \t|\t point = 1.011 \t|\t f(point) = -1.000\n    Iteration: 6 \t|\t point = 0.959 \t|\t f(point) = -0.997\n    Iteration: 7 \t|\t point = 0.991 \t|\t f(point) = -1.000\n    Iteration: 8 \t|\t point = 1.011 \t|\t f(point) = -1.000\n    Iteration: 9 \t|\t point = 0.998 \t|\t f(point) = -1.000\n    Iteration: 10 \t|\t point = 1.006 \t|\t f(point) = -1.000\n    Searching finished. Successfully. code 0\n    1.0059846881033916\n\nModels\n-------\nYou can use our models for classification and regression\n\n.. code-block:: python\n\n    from nueramic_mathml.ml import LogisticRegressionRBF\n    from sklearn.datasets import make_moons\n\n    x, y = make_moons(10_000, noise=.1, random_state=84)\n    x, y = torch.tensor(x), torch.tensor(y)\n    logistic_model_rbf = LogisticRegressionRBF(x[:1000]).fit(x, y, show_epoch=10)\n\n    Epoch:     1 | CrossEntropyLoss:  0.71496\n    Epoch:    12 | CrossEntropyLoss:  0.35328\n    Epoch:    23 | CrossEntropyLoss:  0.27769\n    Epoch:    34 | CrossEntropyLoss:  0.22395\n    Epoch:    45 | CrossEntropyLoss:  0.19266\n    Epoch:    56 | CrossEntropyLoss:  0.16695\n    Epoch:    67 | CrossEntropyLoss:  0.14686\n    Epoch:    78 | CrossEntropyLoss:  0.13051\n    Epoch:    89 | CrossEntropyLoss:  0.11724\n    Epoch:   100 | CrossEntropyLoss:  0.10629\n\n    logistic_model_rbf.metrics_tab(x, y)\n\n    {\'auc_roc\': 0.9974513817072977,\n     \'f1\': 0.9700730618209839,\n     \'precision\': 0.9709476828575134,\n     \'recall\': 0.9692000150680542}',
    'author': 'Victor Barbarich',
    'author_email': 'vktrbr@icloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nueramic/mathml',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.1,<3.11',
}


setup(**setup_kwargs)



from .one_animation import gen_animation_gss, gen_animation_spi, gen_animation_brent
from .multi_animation import make_surface, make_contour, gen_animated_surface, gen_simple_gradient

__all__ = [
    "gen_animation_gss",
    "gen_animation_spi",
    "gen_animation_brent",
    "make_surface",
    "make_contour",
    "gen_animated_surface",
    "gen_simple_gradient"
]
